# インスタンス分割ツール

## 環境

Anaconda3-4.2かそれ以前のバージョン。
Anaconda3-4.3は、(2017/02/25の時点では)OpenCVが提供されていないため動作確認できなかった。

確認したOSとバージョンは以下のとおり。

* MacOS X 10.11.6, Anaconda3-4.2.0
* Ubuntu 16.04.2 LTS, Anaconda3-4.2.0
* Windows 7, Anaconda3-4.2.0

AnacondaにはOpenCVが含まれないため、以下のコマンドでOpenCVをあらかじめインスタンスしておく。

<pre>
$ conda install -c menpo opencv3
</pre>

## 画像データ

画像フォーマットはPNGのみとする。

### ファイル名命名規則

* 原画像ファイル名は"_rgb"を含むこと
* 原画像に対応するラベル画像ファイル名は、"_rgb"を"_label"に置き換えたもの
* フレーム番号は、"_frame"に続く数字とすること
  たとえば、"ara2012_plant01_frame05_rgb.png"のフレーム番号は05となる
 * フレーム番号は不連続でもかまわない

## 操作方法

### 開く

File -> Open... で原画像を選択する。ファイル名命名規則より、"_rgb"を含むもののみが選択できる。
開く際に、対応するラベルが存在するなら、同時に読み込む。
また、前フレームのラベルも存在するなら、それも読み込む。
開くのショートカットはCtrl-O

### プラントセグメンテーション

メインウィンドウ左下のR, G, Bそれぞれのスライダーを動かすことにより、
大ざっぱなプラントセグメンテーションがおこなえる。

### マスク修正

プラントセグメンテーションは単純なしきい値で制御しているため、葉以外の部分を葉と認識したりすることがある。
そのため、マスクの修正が必要となる。

マスクの修正は分割線(後述)と同じ操作でおこなうことができる。
ペンの色を黒もしくは白とするとマスクを直接修正することができる。
周辺に発生するノイズを消去する場合、ブラシサイズを大きくすると効率的にノイズを除去できる。

### 分割線

分割線を引く場合は、あらかじめコンボボックスで色を選択しておく。
仕様により、始点と終点はマスクの黒画素でなければならない。

### 統合

本ソフトウェアの構成では、マスクの上に分割線のレイヤーが存在している。(詳細はビュー構成)
葉の塗り分けをおこなう前に統合をおこなわなければならい。

### 葉の塗り分け

葉の塗り分けは分割線と同じく、あらかじめ色を選んでおく。
そして、白の部分をクリックすると、その閉曲線を単一色で塗りつぶす。

### ズーム, パン

ズームをおこなうには、Controlキーを押しながら、マウスホイールを回転させる。
パンも同様に、Controlキーを押しながらマウスをドラッグさせる。

### アンドゥ, リドゥ

分割線と葉の塗り分けについてはアンドゥをおこなうことができる。
アンドゥをおこなうには、Edit -> Undoもしくは、Ctrl-Z。
リドゥは、Edit -> Redoもしくは、Ctrl-Shift-Z。

### 前フレームの表示

開くの時に前フレームのマスク画像が存在するなら、読み込むよう設計している。
そして、この画像を表示するには、「前フレームを表示」のチェックボックスにチェックをいれる。
前フレームと原画像の表示は排他となっている。

## ビュー構成

上(手前)から順に、

* ペンレイヤー: 分割線
* マスクレイヤー: マスク画像
* 前フレームレイヤー
* 原画像: 原画像

となっている。
ペンレイヤーとマスクレイヤーを分けた理由は、以下のとおり。

* 分割線に対するアンドゥ操作の実装が容易になる
* メモリーやCPU資源的にリーズナブルになる

つまり、ディスプレイ上では分割されているように見えるが、
それぞれは独立に存在しているため、実際には分割されていない状態となっている。
そこで、ペンレイヤーをマスクレイヤーに統合することによりマスクレイヤーを分割することができる。

## leaf.csv

葉の色はleaf.csvで定義する。
インスタンス分割ツールは、起動時にこのファイルを読み込み、コンボボックスに設定する。

leaf.csvの文字コードはutf-8。フォーマットは、以下の通り。

<pre>
  名前,赤の値,緑の値,青の値
</pre>

以下に例を挙げる。

<pre>
leaf1,   0,   0, 255
leaf2,   0, 255,   0
leaf3,   0, 255, 255
leaf4, 255,   0,   0
leaf5, 255,   0, 255
leaf6, 255, 255,   0
マスク,   0,   0,   0
葉,    255, 255, 255
</pre>

## 画面構成

1. プラントセグメンテーション用スライダー類の有効/無効の切り替え
  チェックを外すと、(2)から(8)までが無効となる
2. プラントセグメンテーション用赤のしきい値スライダー
3. プラントセグメンテーション用赤のしきい値スピンボックス
4. プラントセグメンテーション用緑のしきい値スライダー
5. プラントセグメンテーション用緑のしきい値スピンボックス
6. プラントセグメンテーション用青のしきい値スライダー
7. プラントセグメンテーション用青のしきい値スピンボックス
8. 上記RGBスライダーを同期して動かす
9. 分割線の色選択
10. 分割線の太さスライダー
11. 分割線の太さスピンボックス
12. マスクの不透明度。左側が透明度が高くなる
13. 前フレームのマスクの表示。前フレームのマスクと原画は排他的に表示される
  前フレームのマスクが存在しない場合は、グレーアウトする
14. マスクの保存。ファイル名は原画像名の"_rgb"を"_label"に置き換えたものになる
15. 分割線をマスクに統合する

![画面構成](doc/ss.png)
