# coding: utf-8

import os
import re


def createFrameRegex(fname, keyword='_frame', compile=True):
    '''
    ファイル名から前/次フレームの正規表現パターンを返す

    フレーム番号は"_frame"に続く数字とする
    たとえば、ファイル名が"ara2012_plant01_frame13_rgb.png"であれば"13"がフレーム番号となる
    そのため、生成されるパターンは"ara2012_plant01_frame\d+_rgb.png"となる
    '''
    fname = os.path.basename(fname)
    p = r'^(.*%s)(\d+)(.*)$' % keyword
    a = re.split(p, fname)
    a[2] = r'\d+'
    if compile:
        return re.compile(''.join(a))
    return ''.join(a)

def getFiles(fname, keyword='_frame'):
    '''
    fnameと同じディレクトリにあるcreateFrameRegex(fname)にマッチするリストを返す
    戻り値はファイル名を辞書順でソートしたもの
    '''
    dir = os.path.dirname(fname)
    rx = createFrameRegex(fname, keyword)
    files = ['%s/%s' % (dir, f) for f in os.listdir(dir) if rx.match(f)]
    files.sort()
    return files


def getNextFrame(fname, keyword='_frame'):
    '''
    fnameの次フレームのファイル名を返す

    フレーム番号が不連続であっても、実在するファイル名を返す
    例えば['f1', 'f3', 'f5']が存在したときにfname='f3'ならば'f5'を返す
    fname='f5'の場合はNoneを返す
    '''
    a = getFiles(fname, keyword)
    try:
        i = a.index(fname)
        return a[i + 1]
    except ValueError:
        return None
    except IndexError:
        return None


def getPreviousFrame(fname, keyword='rgb_00_04_001_0'):
    '''
    fnameの前フレームのファイル名を返す

    フレーム番号が不連続であっても、実在するファイル名を返す
    例えば['f1', 'f3', 'f5']が存在したときにfname='f3'ならば'f1'を返す
    fname='f1'の場合はNoneを返す
    '''
    a = getFiles(fname, keyword)
    print(a)
    try:
        i = a.index(fname)
        if i == 0:
            return None
        return a[i - 1]
    except ValueError:
        return None
    except IndexError:
        return None


def getNextView(fname):
    return getNextFrame(fname, keyword='_view')


def getPreviousView(fname):
    return getPreviousFrame(fname, keyword='_view')
